//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//-------------------------------------------------------------------------


// AI_ZOMBIE.CPP

#include "game.h"

AIIMAGES aiZombie::aiTiles[] = {
	// TPYE                // Start        // end

	// WALK ANIMATIONS
	{ AIPIC_WALK90L,       1170,           1174 }, // 0
	{ AIPIC_WALK90R,       1175,           1179 }, // 1
	{ AIPIC_WALK90L2,      1180,           1184 }, // 2
	{ AIPIC_WALK90R2,      1185,           1189 }, // 3
	{ AIPIC_WALK90L3,      1190,           1194 }, // 4
	{ AIPIC_WALK90R3,      1195,           1199 }, // 5
	{ AIPIC_WALK90L4,      1200,           1204 }, // 6
	{ AIPIC_WALK90R4,      1205,           1209 }, // 7

	// DAMAGE ANIMATIONS


	{ -1,                    -1,             -1 }
};

/**********************************************/
// Code for Zombies
/**********************************************/

aiZombie::aiZombie()
{
	// Set monster defaults
	health    = 100;
	animCycle = 0;
	footing   = FALSE;
	aiChangeState(AISTATE_IDLE);
}
/*************************************************

Function:void aiZombie::mobChangeAngle(long newang)

Info:
Changes this mobs angle sprite, allows for full
turning etc.
*************************************************/

void aiZombie::mobChangeAngle(long newang)
{
	int walkStyle = 0;
	if(animCycle >= 3)
		animCycle = 0;

	if(newang > 256)
		walkStyle++;
	if(newang > 512)
		walkStyle++;
    if(newang > 768)
		walkStyle++;
    if(newang > 1024)
		walkStyle++;

	switch(footing)
	{
		case FALSE:
			footing = TRUE;
			mesprite->picnum = thisGame->mobChangePic(aiTiles[0].start,aiTiles[2].start,aiTiles[4].start,aiTiles[6].start,animCycle,walkStyle,4);
		break;
		case TRUE:
			footing = FALSE;
			mesprite->picnum = thisGame->mobChangePic(aiTiles[1].start,aiTiles[3].start,aiTiles[5].start,aiTiles[7].start,animCycle,walkStyle,4);
		break;
	}
	animCycle++;
}

/****************************************************
Function: void aiZombie::thinkWalk(void)

Info:
Thinking function for walking.  Does NOT change
sprite animations
****************************************************/


void aiZombie::thinkWalk(void)
{
	long da,dx,dy;
	long hitobj;
	long   cosa, sina;
	short  sect=0, hitsect=0, hitspr=0, hitwall=0;
	long   zvel=0,  hitx=0,    hity=0,   hitz=0;
	bloodPlayer_t *thisPlayer = thisGame->gamePlayers.at(0);
	spritetype    *sprite2;

	sina = (sin((long double)mesprite->ang * (3.141592/1024)) * 16383);
	cosa = (cos((long double)mesprite->ang * (3.141592/1024)) * 16383);

	zvel = (100-48-3)*32;

	build_func->hitscan(mesprite->x,mesprite->y,mesprite->z,mesprite->sectnum,
                cosa,
                sina,
                zvel<<6,&hitsect,&hitwall,&hitspr,&hitx,&hity,&hitz,CLIPMASK1);

	sprite2 = build_func->Game_GetSprite(hitspr);

	dx = thisPlayer->posx - mesprite->x;
	dy = thisPlayer->posy - mesprite->y;

	mesprite->ang = build_func->getangle(dx,dy);

	hitobj = thisGame->MoveSprite(mesprite,20,0,CLIPMASK0);
	if(hitobj != 0)
	{
		mesprite->ang += 320;
		thisGame->MoveSprite(mesprite,-20,0,CLIPMASK0);
		mobChangeAngle(513);
	}
	else
	{
		// see if we hit a sprite.
		if(sprite2 != NULL)
		{
			if(FindDistance2D(mesprite->x-sprite2->x,mesprite->y-sprite2->y) < 3024)
			{
				mesprite->ang += 320;
				thisGame->MoveSprite(mesprite,-20,0,CLIPMASK0);
				mobChangeAngle(513);
			}
			else
			{
				mobChangeAngle(1);
			}
		}
		else
		{
				mobChangeAngle(1);
		}
	}
}

/**************************************************

Function: void aiZombie::thinkAttack(void)

Info: Never going to guess what this function does...
**************************************************/


void aiZombie::thinkAttack(void)
{
	if(animCycle > 2)
		animCycle = 0;

	if(mesprite->picnum == 1234)
		soundManager->soundPlay(21);
	else
		thisGame->MobAttackPlayer(2>>1);
	

	mesprite->picnum = 1234 + (animCycle * 10);	

	animCycle++;
}

/*************************************************

Function:ThinkMain

Info:
The local main for AI, loops untill death.  This is
the only function called by thread system
*************************************************/

void aiZombie::thinkMain(void)
{
	long oldz;
	bloodPlayer_t     *thisPlayer = thisGame->gamePlayers.at(0);
	gameSprites_t     *thisSprite = new gameSprites_t;

	mesprite  = build_func->Game_GetSprite(this->spritenum);
	
	thisSprite->health    = 100;
	thisSprite->pain      = 0;
	thisSprite->spriteNum = this->spritenum;

	thisGame->gameSprites.push_back(thisSprite);

	// Thread loop
	while(thisSprite->health > 0)
	{
		if(aiGetState() == AISTATE_IDLE) // actor hasn't seen the player at least once.
		{
			if(mesprite->type == 205)
			{	
				mesprite->picnum = 3046;
				mesprite->z      = build_func->GetSector(mesprite->sectnum)->floorz;
				if(FindDistance2D(thisPlayer->posx-mesprite->x,thisPlayer->posy-mesprite->y) < 3024)
					aiChangeState(AISTATE_RAISE);
			}
			else if(mesprite->type == 203 && common->spriteSeePlayer(this->spritenum))
			{
				soundManager->soundPlay(9);
				aiChangeState(AISTATE_SEARCH);
			}
		}
		else if(thisSprite->pain == 1) // something hit me, damage is done by caller
		{
			mesprite->picnum = 1200;
			thisSprite->pain = 0;
			soundManager->soundPlay(10);
			aiChangeState(AISTATE_SEARCH);
		}
		else if(aiGetState() == AISTATE_RAISE) // actor raising up from the ground.
		{
			if(mesprite->picnum >= 3054)
			{
				soundManager->soundPlay(9);
				aiChangeState(AISTATE_SEARCH);
			}
			else
			{
				mesprite->picnum++;
				mesprite->z     -= 1000;
			}
		}
		else // default AI, attack or move...complicated isn't it?
		{		
			if(FindDistance2D(thisPlayer->posx-mesprite->x,thisPlayer->posy-mesprite->y) > 1024)
			{
				thinkWalk();
			}
			else
			{
				thinkAttack();
			}
		}
		Sleep(90); // let windows functions go through
	}

	// Play death animation and quit thread
	soundManager->soundPlay(5);
	for(int i = 1216; i < 1221; i++)
	{
          mesprite->picnum = i;  
		  mesprite->z     += 1000;
          Sleep(40);
	}
	mesprite->picnum = 1221;
	thisGame->makeitfall(spritenum);
}