//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//-------------------------------------------------------------------------


// game_actor.cpp

#include "game.h"

aiConstruct Game::activeMonsters[] = {
	{ 1170, 1258, 3054, 3054, NULL,MONSTER_ZOMBIE },

	{ -1,     -1,   -1,   -1, NULL, NULL }
};

/********************************************

Function: void Game::InternalRegisterAI(void)

Info: Registers all AI in the map

********************************************/

void Game::InternalRegisterAI(void)
{
	spritetype *thisSprite;
	int        thisAI=0;
	while( activeMonsters[thisAI].beginPic != -1 )
	{
		for(int i = 0; i < this->numofsprites; i++)
		{
			thisSprite = build_func->Game_GetSprite(i);

			if(thisSprite == NULL)
				continue;

			if(thisSprite->picnum >= activeMonsters[thisAI].beginPic &&
			thisSprite->picnum <= activeMonsters[thisAI].endPic)
			{
				activeMonsters[thisAI].mesprite = i;
				ai_SpawnThread(activeMonsters[thisAI]);
			}
			else if(thisSprite->picnum >= activeMonsters[thisAI].beginPic2 &&
					thisSprite->picnum <= activeMonsters[thisAI].endPic2)
			{
				activeMonsters[thisAI].mesprite = i;
				ai_SpawnThread(activeMonsters[thisAI]);
			}
		}
		thisAI++;
	}
}

/*
=======================================================================================
int Game::mobChangePic(int baseFrame1, int baseFrame2, int baseFrame3,int baseFrame4,
					   int cycle, int framenum, int maxframes)
=======================================================================================
*/

int Game::mobChangePic(int baseFrame1, int baseFrame2, int baseFrame3,int baseFrame4,
					   int cycle, int framenum, int maxframes)
{

	switch(cycle)
	{
		case 0:
			return baseFrame1 + framenum;
		break;
		case 1:
			return baseFrame2 + framenum;
		break;
		case 2:
			return baseFrame3 + framenum;
		break;
		case 3:
			return baseFrame4 + framenum;
		break;
	}
}

/*
==============================================================
boolean Game::mobCheckMove(spritetype *thisSprite, int dist)
==============================================================
*/

boolean Game::mobCheckMove(spritetype *thisSprite, int dist)
{
	return FALSE;
}

/*
======================================================================
long Game::MoveSprite(spritetype *thisSprite, long fvel, long svel, unsigned long cliptype)

Moves a sprite with clipping
======================================================================
*/

long Game::MoveSprite(spritetype *thisSprite, long fvel, long svel, unsigned long cliptype)
{
	long avel=0;
	long cosa,sina;
	long momx=0,momy=0;
	long globhiz=0, globhihit=0, globloz=0,globlohit=0,goalz=0;
	long retval;
	momx = mulscale9(fvel,build_func->sintable[(thisSprite->ang+2560)&2047]);
    momy = mulscale9(fvel,build_func->sintable[(thisSprite->ang+2048)&2047]);
	
	if(fvel != 0)
	{
		sina = (sin((long double)thisSprite->ang * (3.141592/1024)) * 16383);
		cosa = (cos((long double)thisSprite->ang * (3.141592/1024)) * 16383);

		momx += ((((long)fvel)*40*cosa)>>3);
		momy += ((((long)fvel)*40*sina)>>3);
	}

	if(svel != 0)
	{
		sina = (sin((long double)thisSprite->ang * (3.141592/1024)) * 16383);
		cosa = (cos((long double)thisSprite->ang * (3.141592/1024)) * 16383);

		momx += ((((long)svel)*5*cosa)>>3);
		momy += ((((long)svel)*5*sina)>>3);
	}

	retval = build_func->clipmove(&thisSprite->x,&thisSprite->y,&thisSprite->z,&thisSprite->sectnum,momx,momy,192L,4<<8,4<<8,cliptype);
	build_func->getzrange(thisSprite->x,thisSprite->y,thisSprite->z,thisSprite->sectnum,&globhiz,&globhihit,&globloz,&globlohit,128L,cliptype);
	goalz = globloz-10000;
	thisSprite->z = goalz;

	return retval;
}

/*
=======================================
void Game::makeitfall(int spriteNum)
=======================================
*/

void Game::makeitfall(int spriteNum)
{
	spritetype    *thisSprite    = build_func->Game_GetSprite(spriteNum);
	sectortype    *sprSector     = build_func->GetSector(thisSprite->sectnum);

	thisSprite->z = sprSector->floorz-2000;
}

/*
===================================================
void Game::MobExecuteAI(int aiNum, int spritenum)

TODO:
Move me to scripts
===================================================
*/
void Game::MobExecuteAI(int aiNum, int spritenum)
{
	bloodPlayer_t *thisPlayer    = gamePlayers.at(0);
	spritetype    *thisSprite    = build_func->Game_GetSprite(spritenum);
	long          fvel=0,svel=0;
	char temptxt[100];

	switch(aiNum)
	{
		case AI_ZOMBIEMOVE:
			if(thisSprite->picnum > 1190 || thisSprite->picnum < 1170)
				thisSprite->picnum = 1170;

			// TODO: CAUSES FATAL CRASH
			thisSprite->picnum += 10;

			thisSprite->ang = thisPlayer->ang;
			fvel -= 26;  // zombie movement speed
			MoveSprite(thisSprite,fvel,svel,CLIPMASK0);
			Sleep(100);
		break;

		default:
			sprintf(temptxt,"MobExecuteAI: %d not valid\n",aiNum);
			FatalCrash(temptxt);
		break;
	}
}

/*
=======================================
void Game::MobAttackPlayer(int damage)
=======================================
*/

void Game::MobAttackPlayer(int damage)
{
	if(gamePlayers.at(0)->health <= 0)
	{
		gamePlayers.at(0)->health = 0;
		return;
	}

	if(damage <= 10 && damage > 0)
	{
		soundManager->soundPlay(11);
	}
	else if(damage <= 20 && damage > 10)
	{

	}
	else if(damage > 20)
	{

	}
	gamePlayers.at(0)->health -= damage;
}

/*
===========================================
boolean Game::MobInMeleeRange(int sprNum)
===========================================
*/

boolean Game::MobInMeleeRange(int sprNum)
{
	bloodPlayer_t *thisPlayer = gamePlayers.at(0);
	spritetype    *thisMob    = build_func->Game_GetSprite(sprNum);
	long   cosa, sina;
	short  sect=0, hitsect=0, hitspr=0, hitwall=0;
	long   zvel=0,  hitx=0,    hity=0,   hitz=0;
	char temptxt[100];

	if(thisMob == NULL)
		exit(0);

	sina = (sin((long double)thisMob->ang * (3.141592/1024)) * 16383);
	cosa = (cos((long double)thisMob->ang * (3.141592/1024)) * 16383);

	zvel = (100-48-3)*32;

	build_func->hitscan(thisMob->x,thisMob->y,thisMob->z,thisMob->sectnum,
                cosa,
                sina,
                zvel<<6,&hitsect,&hitwall,&hitspr,&hitx,&hity,&hitz,CLIPMASK1);

	if(FindDistance2D((long)thisPlayer->posx-hitx, (long)thisPlayer->posy-hity) < 1024)
	{
	//	if(thisMob->sectnum == thisPlayer->sectorNum)
			return 1;
	}
	
	return 0;
}

/*
===============================================================================
void  Game::entWonder(int spriteNum, int tics, int frontWalk, int frontWalk2)
===============================================================================
*/

void  Game::entWonder(int spriteNum, int tics, int frontWalk, int frontWalk2)
{
	spritetype    *thisSprite = build_func->Game_GetSprite(spriteNum);
	long fvel=0, svel=0, avel=0;
	long cosa,sina;
	long momx=0,momy=0;
	long globhiz=0, globhihit=0, globloz=0,globlohit=0,goalz=0;

	for(int i = 0; i < tics; i++)
	{
		fvel = (build_func->krand()&255)-128;
		//svel = (build_func->krand()&255)-128;
		
		momx = mulscale9(fvel,build_func->sintable[(thisSprite->ang+2560)&2047]);
	    momy = mulscale9(fvel,build_func->sintable[(thisSprite->ang+2048)&2047]);
	
		if(svel <= 0)
			svel = min(1+2*40,0);
		if(fvel != 0)
		{
			sina = (sin((long double)thisSprite->ang * (3.141592/1024)) * 16383);
			cosa = (cos((long double)thisSprite->ang * (3.141592/1024)) * 16383);

			momx += ((((long)fvel)*40*cosa)>>3);
			momy += ((((long)fvel)*40*sina)>>3);
		}

		if(svel != 0)
		{
			sina = (sin((long double)thisSprite->ang * (3.141592/1024)) * 16383);
			cosa = (cos((long double)thisSprite->ang * (3.141592/1024)) * 16383);

			momx += ((((long)svel)*5*cosa)>>3);
			momy += ((((long)svel)*5*sina)>>3);
		}

		thisSprite->picnum = frontWalk;
		Sleep(40);
		build_func->clipmove(&thisSprite->x,&thisSprite->y,&thisSprite->z,&thisSprite->sectnum,momx,momy,128L,4<<8,4<<8,CLIPMASK0);
		build_func->getzrange(thisSprite->x,thisSprite->y,thisSprite->z,thisSprite->sectnum,&globhiz,&globhihit,&globloz,&globlohit,128L,CLIPMASK0);
		goalz = globloz-10000;
		thisSprite->z = goalz;
		thisSprite->picnum = frontWalk2;
		Sleep(40);
	}
}

