//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// script_runner.cpp

#include "game.h"
#include "script_runner.h"

#define SCRIPT_COMMENT 0
#define SCRIPT_INCLUDE 1
#define SCRIPT_VOID    2
#define SCRIPT_ENDOF   3
#define SCRIPT_OPENP   4
#define SCRIPT_CLOSEP  5
#define SCRIPT_OPENB   6
#define SCRIPT_CLOSEB  7

#define EVENT_MAIN	   0  // entry point into the script

#define IF_WEAPONTILE  1
#define IF_KEYDOWN     2
#define IF_RAND        3
#define IF_RELOADTIME  4
#define IF_THISHEALTH  5
#define IF_CANSEEME    6
#define IF_MELEERANGE  7
#define IF_SPOTTED     8
#define IF_PAIN        9

#define IFTYPE_EQUAL   1
#define IFTYPE_LESS    2
#define IFTYPE_GREATER 3

char *script_runner::punctuation[] = {
	"//", "include", "void", "end_of_script",
	"(", ")", "{", "}"
};

char *script_runner::events[] = {
	"main"
};

/*
==========================================
void script_runner::RunCodeBlock(int start, int end)
==========================================
*/

void script_runner:: RunCodeBlock(int start, int end)
{
	int endPos      = end;
	currentPosition = start;
	char lastPrintf[300];
	vector<int> theseParam;

	while(currentPosition < endPos)
	{
		theseParam.clear();
		//printf("currentPosition = %d, %s\n",currentPosition,compiledProg.at(currentPosition));
		if(!strcmp((char *)compiledProg.at(currentPosition),"setWeaponTile"))
		{
		//	thisGame->thisWeaponPic = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"register_weapon"))
		{
			if(leave_program == false)
				thisGame->Register_Weapon((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"setClientWeaponM"))
		{
			thisGame->SetPlayerWeapon(atoi((char *)compiledProg.at(currentPosition+1)));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"if"))
		{
			ExecConditional();
			if(gotoEndofScript == true)
			{
				gotoEndofScript = false;
				return;
			}
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"EndLoadingPlague"))
		{
			thisGame->EndLoadingPlague();
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"register_ai"))
		{
			int startFrame = atoi((char *)compiledProg.at(currentPosition+1));
			int endFrame   = atoi((char *)compiledProg.at(currentPosition+2));
			char *script   = (char *)compiledProg.at(currentPosition+3);

			thisGame->RegisterAI(startFrame,endFrame,script);

			currentPosition += 2;
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"shutdown_thread"))
		{
			delete this;
			ExitThread(0);
			leave_program = true;
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"removetile"))
		{
			thisGame->removeSprites.push_back(atoi((char *)compiledProg.at(currentPosition+1)));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"endif"))
		{
			//printf("WARNING: endif with out watching IF statement\n");
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"sleepg"))
		{
			int waitAmt = atoi((char *)compiledProg.at(currentPosition+1));
			int times = waitAmt / 30;
			for(int i = 0; i < times; i++)
			{
				guimanager->Blood_Printf(lastPrintf,4449,-1,6,6,10,10);
				Sleep(30);
			}
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"system.rand"))
		{
			thisRand = rand() % atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"print"))
		{
			//thisRand = rand() % atoi((char *)compiledProg.at(currentPosition+1));
			sprintf(lastPrintf,(char *)compiledProg.at(currentPosition+1));
			guimanager->Blood_Printf((char *)compiledProg.at(currentPosition+1),4449,-1,6,6,10,10);
			build_func->nextpage();
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"precacheSound"))
		{
//			soundManager->precache_Sound((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"int"))
		{
			variable_t *thisVar;
			currentPosition++;
            
			thisVar = new variable_t((char*)compiledProg.at(currentPosition),"0");
			scriptVariables.push_back(thisVar);
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gthisWeaponY"))
		{
			thisGame->thisWeaponY   = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gsetClientWeapon"))
		{
			thisGame->SetPlayerWeapon(atoi((char *)compiledProg.at(currentPosition+1)));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gthisWeaponX"))
		{
			thisGame->thisWeaponX   = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gaddOneToReload"))
		{	
			if(thisReloadAmount < 0)
				thisReloadAmount = 0;
			else
				thisReloadAmount+=1;
		}	
		else if(!strcmp((char *)compiledProg.at(currentPosition),"grefreshReload"))
		{	
			thisReloadAmount = 0;
		}	
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gplaysound"))
		{	
			soundManager->soundPlay(atoi((char *)compiledProg.at(currentPosition+1)));
			//soundManager->playsnd3d(this->spriteNum,atoi((char *)compiledProg.at(currentPosition+1)));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gsetPicNum"))
		{
			try
			{
				spritetype *thisSprite = build_func->Game_GetSprite(this->spriteNum);
			
				thisSprite->picnum     = atoi((char *)compiledProg.at(currentPosition+1));
			}
			catch(...)
			{
			}
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"NodrawWeapon"))
		{
			thisGame->NodrawWeapon = TRUE;
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gsetWeaponTile"))
		{
			thisGame->thisWeaponPic = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gthisWeaponY2"))
		{
			thisGame->thisWeaponY2   = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gthisWeaponX2"))
		{
			thisGame->thisWeaponX2   = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"gsetWeaponTile2"))
		{
			thisGame->thisWeaponPic2 = atoi((char *)compiledProg.at(currentPosition+1));
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"startAI"))
		{
			thisGame->StartAI();
		}
		else if(!strcmp((char *)compiledProg.at(currentPosition),"SpriteAnimation"))
		{
			

			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+1)));
			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+2)));
			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+3)));
			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+4)));
			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+5)));
			theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+6)));

			currentPosition += theseParam.size() - 1;

			thisGame->SpriteAnimation(theseParam.at(0),theseParam.at(1),
									   theseParam.at(2),theseParam.at(3),
									   theseParam.at(4),theseParam.at(5));
		}
		else // function or variable
		{
			token = (char *)compiledProg.at(currentPosition);
			for(int i = 0; i < scriptVariables.size(); i++)
			{
			//	printf("token = %d:%s %d:%s\n",strlen(token),token,strlen(scriptVariables.at(i)->name),scriptVariables.at(i)->name);
				if(strcmp(token, scriptVariables.at(i)->name))
				{
					currentPosition++;
					token = (char *)compiledProg.at(currentPosition);

					scriptVariables.at(i)->val++;
					
					break;
				}
			}

			//if( i == scriptVariables.size())
			//	printf("WARNING: Invalid or unexpected token %s\n",token);
		}
		currentPosition++;
	}
	scriptVariables.clear();
	//printf("LEAVING LOOP\n");
}

/*
===========================================
void script_runner::ExecConditional(void)
===========================================
*/

void script_runner::ExecConditional(void)
{
	int conditional_type  = -1;
	int conditional_value = -1;
	int cmp_value         = -1;
	int inttmp;
	char *token;
	boolean ignoreConditional = false;
	vector<int> theseParam;

	token = (char *)compiledProg.at(currentPosition);

	if(!strcmp(token,"if"))
		currentPosition++; // skip IF
	while( 1 )
	{
		theseParam.clear();
		token = (char *)compiledProg.at(currentPosition);

		if(!strcmp(token,"system.weapontile"))
		{
			conditional_value = IF_WEAPONTILE;
		}
		else if(!strcmp(token,"system.keydown"))
		{
			conditional_value = IF_KEYDOWN;
		}
		else if(!strcmp(token,"system.cansee"))
		{
			conditional_value = IF_CANSEEME;
		}
		else if(!strcmp(token,"system.health"))
		{
			conditional_value = IF_THISHEALTH;
		}
		else if(!strcmp(token,"system.meleerange"))
		{
			conditional_value = IF_MELEERANGE;
		}
		else if(!strcmp(token,"system.spotted"))
		{
			conditional_value = IF_SPOTTED;
		}
		else if(!strcmp(token,"system.pain"))
		{
			conditional_value = IF_PAIN;
		}
		else if(!strcmp(token,"system.lastrand"))
		{
			conditional_value = IF_RAND;
		}
		else if(!strcmp(token,"system.reloadTime"))
		{
			conditional_value = IF_RELOADTIME;
		}
		else
		{
			printf("WARNING: Invalid conditional statement with value %s\n",token);
			return;
		}
		currentPosition++;
		token = (char *)compiledProg.at(currentPosition);

		if(!strcmp(token,"=="))
		{
			conditional_type = IFTYPE_EQUAL;
		}
		else if(!strcmp(token,">="))
		{
			conditional_type = IFTYPE_GREATER;
		}
		else if(!strcmp(token,"<="))
		{
			conditional_type = IFTYPE_GREATER;
		}
		else
		{
			printf("WARNING: Invalid conditional type %s\n",token);
			return;
		}
		
		currentPosition++;
		token = (char *)compiledProg.at(currentPosition);

		cmp_value = atoi(token);

		currentPosition++;
		token = (char *)compiledProg.at(currentPosition);

		if(!strcmp(token,"then"))
		{
			printf("WARNING: IF expected 'then', but found %s\n",token);
			return;
		}

        switch(conditional_value)
		{
			case IF_PAIN:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						ignoreConditional = true;
						for(int i = 0; i < thisGame->gameSprites.size(); i++)
						{
							if(thisGame->gameSprites.at(i)->spriteNum == this->spriteNum)
							{
								if(thisGame->gameSprites.at(i)->pain == cmp_value)
								{
									inttmp = i;
									ignoreConditional = false;
									break;
								}
							}
						}
					break;
				}
			break;
			case IF_SPOTTED:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						ignoreConditional = true;
						if(playerspoted == cmp_value)
							ignoreConditional = false;
					break;
				}
			break;
			case IF_MELEERANGE:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						ignoreConditional = true;
						if(thisGame->MobInMeleeRange(spriteNum) == cmp_value)
							ignoreConditional = false;
					break;
				}
				break;
			break;
			case IF_CANSEEME:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						ignoreConditional = true;
						if(common->spriteSeePlayer(spriteNum) == cmp_value)
						{
							ignoreConditional = false;
						}
					break;
				}
			break;
			case IF_THISHEALTH:
				switch(conditional_type)
				{
					case IFTYPE_GREATER:
						ignoreConditional = true;
						//printf("checking to see if i am hit\n");
						for(int i = 0; i < thisGame->gameSprites.size(); i++)
						{
							if(thisGame->gameSprites.at(i)->spriteNum == this->spriteNum)
							{
								if(thisGame->gameSprites.at(i)->health < cmp_value)
								{
									ignoreConditional = false;
								}
							}
						}
					break;
				}
			break;
			case IF_RELOADTIME:
				switch(conditional_type)
				{
					case IFTYPE_GREATER:
						if(thisReloadAmount < cmp_value)
						{
							ignoreConditional = true;
						}
					break;
				}
			break;
			case IF_WEAPONTILE:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						if(thisGame->thisWeaponPic != cmp_value)
							ignoreConditional = true;
					break;
				}
			break;

			case IF_RAND:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						if(thisRand != cmp_value)
							ignoreConditional = true;
					break;

					case IFTYPE_LESS:
						if(thisRand >= cmp_value)
							ignoreConditional = true;
					break;
					case IFTYPE_GREATER:
						if(thisRand <= cmp_value)
							ignoreConditional = true;
					break;
				}
			break;

			case IF_KEYDOWN:
				switch(conditional_type)
				{
					case IFTYPE_EQUAL:
						if(!build_func->build_KeyDown(cmp_value))
							return;
					break;
				}
			break;
		}

		// Code inside of the IF statement
		while( 1 )
		{
			currentPosition++;
			token = (char *)compiledProg.at(currentPosition);

			if(ignoreConditional == true)
			{
				if(token == NULL)
					return;

				if(!strcmp(token,"endif"))
				{
					return;
				}
				continue;
			}

			if(token == NULL)
				return;

			if(!strcmp(token,"endif"))
			{
				return;
			}
			else if(!strcmp(token,"setpain"))
			{
				thisGame->gameSprites.at(inttmp)->pain = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"spotted"))
			{
				playerspoted = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"makeitfall"))
			{
				thisGame->makeitfall(spriteNum);
			}
			else if(!strcmp(token,"MobExecuteAI"))
			{
				thisGame->MobExecuteAI(atoi((char *)compiledProg.at(currentPosition+1)),spriteNum);
			}
			else if(!strcmp(token,"damagePlayer"))
			{
				thisGame->MobAttackPlayer(atoi((char *)compiledProg.at(currentPosition+1)));
			}
			else if(!strcmp(token,"projectilesize"))
			{
			    thisGame->forcedProjWidth  = atoi((char *)compiledProg.at(currentPosition+1));
				thisGame->forcedProjHeight = atoi((char *)compiledProg.at(currentPosition+2));

				currentPosition += 1;
			}
			else if(!strcmp(token,"PlayerFireWeapon"))
			{
				

				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+1)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+2)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+3)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+4)));

				currentPosition += theseParam.size() - 1;

				thisGame->PlayerFireWeapon(theseParam.at(0),theseParam.at(1),
										   theseParam.at(2),theseParam.at(3),-1);
			}
			else if(!strcmp(token,"PlayerFireWeapon1"))
			{
				

				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+1)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+2)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+3)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+4)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+5)));

				currentPosition += theseParam.size() - 1;

				thisGame->PlayerFireWeapon(theseParam.at(0),theseParam.at(1),
										   theseParam.at(2),theseParam.at(3),theseParam.at(4));
			}
			else if(!strcmp(token,"thisWeaponY"))
			{
				thisGame->thisWeaponY   = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"moveSectorZ"))
			{
				
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+1)));
				theseParam.push_back(atoi((char *)compiledProg.at(currentPosition+2)));

//				thisGame->MoveSector(theseParam.at(0),theseParam.at(1));

				printf("Moving sector %d to %d\n",theseParam.at(0),theseParam.at(1));
				currentPosition += theseParam.size() - 1;
			}
			else if(!strcmp(token,"setEntHealth"))
			{
				for(int i = 0; i < thisGame->gameSprites.size(); i++)
				{
					if(thisGame->gameSprites.at(i)->spriteNum == this->spriteNum)
					{
						thisGame->gameSprites.at(i)->health = atoi((char *)compiledProg.at(currentPosition+1));
						break;
					}
				}
			}
			
			else if(!strcmp(token,"setClientWeapon"))
			{
				thisGame->SetPlayerWeapon(atoi((char *)compiledProg.at(currentPosition+1)));
			}
			else if(!strcmp(token,"hurtplayer"))
			{
				thisGame->gamePlayers.at(0)->health -= atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"thisWeaponX"))
			{
				thisGame->thisWeaponX   = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"addOneToReload"))
			{	
				if(thisReloadAmount < 0)
					thisReloadAmount = 0;
				else
					thisReloadAmount+=1;
			}	
			else if(!strcmp(token,"refreshReload"))
			{	
				thisReloadAmount = 0;
			}	
			else if(!strcmp(token,"playsound"))
			{	
				soundManager->soundPlay(atoi((char *)compiledProg.at(currentPosition+1)));
			}
			else if(!strcmp(token,"entityWonder"))
			{	
				int tics = atoi((char *)compiledProg.at(currentPosition+1));
				int pic1 = atoi((char *)compiledProg.at(currentPosition+2));
				int pic2 = atoi((char *)compiledProg.at(currentPosition+3));

				currentPosition += 2;
				thisGame->entWonder(spriteNum,tics,pic1,pic2);
			}
			else if(!strcmp(token,"setPicNum"))
			{
				try
				{
					spritetype *thisSprite = build_func->Game_GetSprite(this->spriteNum);
				
					thisSprite->picnum     = atoi((char *)compiledProg.at(currentPosition+1));
				}
				catch(...)
				{

				}
			}
			else if(!strcmp(token,"clearKey"))
			{
				build_func->build_ClearKey(atoi((char *)compiledProg.at(currentPosition+1)));
			}
			else if(!strcmp(token,"shootWeapon"))
			{
				int damage = atoi((char *)compiledProg.at(currentPosition+1));
				thisGame->ShotWeapon(damage);
			}
			else if(!strcmp(token,"mobrunai"))
			{
			//	thisGame->MobMain(this->spriteNum);
			}
			else if(!strcmp(token,"setWeaponTile"))
			{
				thisGame->thisWeaponPic = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"thisWeaponY2"))
			{
				thisGame->thisWeaponY2   = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"thisWeaponX2"))
			{
				thisGame->thisWeaponX2   = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"setWeaponTile2"))
			{
				thisGame->thisWeaponPic2 = atoi((char *)compiledProg.at(currentPosition+1));
			}
			else if(!strcmp(token,"sleep"))
			{
				Sleep(atoi((char *)compiledProg.at(currentPosition+1)));
			}
			else if(!strcmp(token,"if"))
			{
				ExecConditional();
			}
			else if(!strcmp(token,"return"))
			{
				gotoEndofScript = true;
				return;
			}
		}

		return;
	}

}

/*
===================================
void script_runner::Execute(void)
===================================
*/

void script_runner::Execute(void)
{
	char *progBuffer  = (char *)this->program.c_str();
	int  progSize;
	gameSprites_t     *thisSprite = new gameSprites_t;

	try
	{
		progSize = strlen(progBuffer);
	}
	catch(...)
	{
		return;
	}

	progEntry        = -1;
	progEnd          = -1;
	thisReloadAmount = 0;
	unfire           = 0;
	playerspoted     = 0;
	leave_program    = false;
	gotoEndofScript  = false;

	thisSprite->health    = 100;
	thisSprite->pain      = 0;
	thisSprite->spriteNum = this->spriteNum;

	thisGame->gameSprites.push_back(thisSprite);

	try
	{
		if(compiledProg.size() <= 0) // compile the program
			CompileScript(progBuffer,progSize);

		thisProgBuffer    = progBuffer;
		while( leave_program == false ) // excecute the program
		{
			compiled_position++;
			if(compiled_position <= 1)
			{
				compiled_position = 2;
				while(compiled_position != 1)
				{
					if(CheckToken(events[EVENT_MAIN]))
					{
						progEntry = compiled_position;

						while( 1 ) // Locate the end of main
						{
							if(CheckToken("}"))
							{
								progEnd = compiled_position;
								break;
							}
							if(compiled_position >= compiledProg.size() || compiled_position < 2)
							{
								printf("ERROR: Main does not have a '}'\n");
								return;
							}
	#if 0
							else
							{
								printf("compiled_prog = %d\n",compiled_position);
							}
	#endif
							compiled_position++;
						}
						//printf("Prog space %d -> %d\n", progEntry, progEnd);

						while( leave_program == false ) // Run Main
						{
							RunCodeBlock(progEntry, progEnd);
							Sleep(5);
						}
						return; // Script ended
					}
					
					if(compiled_position >= compiledProg.size())
					{
						printf("ERROR: Script does not have a entry point\n");
						return;
					}
					compiled_position++;
				}
			}
		}
	}
	catch(...)
	{

	}
}

/*
====================================================
void CompileScript(char *progBuffer, int progSize)

Compile the script.  All this means is assign numbers to
tokens based on what they are for faster and neater lookup.

TODO: This would be a nice place to put syntax checking
====================================================
*/

void script_runner::CompileScript(char *progBuffer, int progSize)
{
	char *token;

	thisProgBuffer = progBuffer;

	currentPosition = 0;
	while(progSize > currentPosition)
	{
		if(currentPosition == 0)
			token = strtok( progBuffer, " \t\n" );
		else
			token = strtok( NULL," \t\n");

		compiledProg.push_back((int)token);
		currentPosition += sizeof(token);
	}
}

/*
===============================================
boolean script_runner::CheckToken(char *tokenIn)
===============================================
*/

boolean script_runner::CheckToken(char *tokenIn)
{
	char *thisToken = (char *)compiledProg.at(compiled_position-1);
	if(thisToken == NULL)
	{
		compiled_position = 1;
		return false;
	}
	//printf("Read token %s size = %d\n", thisToken,strlen(thisToken));
	return (!strcmp(thisToken,tokenIn));
}

/*
======================================
void script_runner::ReadComment(void)
======================================
*/

void script_runner::ReadComment(void)
{
	printf("Found a comment\n");
	while( 1 ) // Find the next newline
	{
		if(thisProgBuffer[currentPosition] == '\n')
		{
			printf("Found endline %d\n",currentPosition);
			return;
		}
		else
		{
			currentPosition++;
		}
	}
}