//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// threadManager.cpp

#include "game.h"
#include "script_runner.h"

char     *progBuffer;
int      spriteNum1 = -1;
/*
======================================
thread_t::thread_t()
======================================
*/

thread_t::thread_t(program_t *thisFunc)
{
	this->thisFunc = thisFunc;

	if(thisFunc->progSize < 5)
	{
		printf("threadManager: Invalid program size < 5\n");
		delete this;
		return;
	}

	threadRunning = false;
}

/*
================================================
void thread_t::ThreadCallFunc(string program)
================================================
*/
DWORD WINAPI ThreadCallFunc1( LPVOID lpParam ) 
{
	char *thisBuffer = progBuffer;
	script_runner    *thisProgram = new script_runner(progBuffer);

	srand(time(NULL));
	thisProgram->spriteNum = *(int*)lpParam;
	try
	{
		thisProgram->Execute();
	}
	catch(...)
	{

	}

	return NULL;
}

/*
================================================
void thread_t::ModifyProgram(char *newProgram)
================================================
*/

void thread_t::ModifyProgram(char *newProgram)
{
	progBuffer = newProgram;
}

/*
=========================================
char *thread_t::GetCurrentProgram(void)
=========================================
*/

char *thread_t::GetCurrentProgram(void)
{
	return progBuffer;
}
/*
================================
void thread_t::RunProgram1(void)
================================
*/
void thread_t::RunProgram1(void)
{
	char *thisBuffer = (char *)thisFunc->progBuffer.c_str();
	script_runner    *thisProgram = new script_runner(progBuffer);

	srand(time(NULL));
	thisProgram->spriteNum = this->thisEntNum;
	thisProgram->Execute();
}

/*
================================
void thread_t::RunProgram(void)
================================
*/

void thread_t::RunProgram(void)
{
	progBuffer = (char *)thisFunc->progBuffer.c_str();

	if(threadRunning == false)
	{
		threadRunning = true;
	}
	else
	{
		ResumeThread(hThread);
		return;
	}
	
	hThread    = CreateThread(NULL,0,ThreadCallFunc1,&thisEntNum,0,&dwThreadId);               
 
	// Check the return value for success. 
	 
	if (hThread == NULL) 
	{
		printf( "RunProgram: CreateThread failed %d\n", GetLastError() ); 
	}
}

/*
=====================================
void thread_t::WaitFor_Thread(void)
=====================================
*/

void thread_t::WaitFor_Thread(void)
{
	WaitForSingleObject(hThread,INFINITE);
}

/*
=================================
void thread_t::KillThread(void)
=================================
*/

void thread_t::KillThread(void)
{
	TerminateThread(this->hThread,0);
}

/*
=====================================
void thread_t::ShutdownThread(void)
=====================================
*/

void thread_t::ShutdownThread(void)
{
	threadRunning = false;
	SuspendThread(hThread);
}


